﻿using System;
using System.Linq;
using UnityEngine;
using UnityEngine.UI;

[RequireComponent(typeof(Button))]
public class ItemButton : MonoBehaviour
{
    public OwnedItemsData.OwnedItem OwnedItem
    {
        get => _ownedItem;
        set
        {
            _ownedItem = value;
            Refresh();
        }
    }

    [SerializeField] private ItemTypeSpriteMap[] itemSprites; // 各アイテム用の画像を指定するフィールド
    [SerializeField] private Image image;
    [SerializeField] private Text number;

    private Button _button;
    private OwnedItemsData.OwnedItem _ownedItem;

    private void Awake()
    {
        _button = GetComponent<Button>();
        _button.onClick.AddListener(OnClick);
    }

    private void OnClick()
    {
        // アイテムボタンを押したときの処理
        switch (_ownedItem.Type)
        {
            case Item.ItemType.Kinoko:
                // きのこはボタンを押すと食べる
                if (_ownedItem.Number > 0)
                {
                    _ownedItem.Use();
                    AudioManager.Instance.Play("eat");
                    MainSceneController.Instance.EatItem();
                    OwnedItemsData.Instance.Save();
                    Refresh();
                }

                break;
        }
    }

    private void Refresh()
    {
        // アイテムが割り当てられたかどうかでアイテム画像や所持個数の表示を切り替える
        var isEmpty = null == _ownedItem;
        image.gameObject.SetActive(!isEmpty);
        number.gameObject.SetActive(!isEmpty);
        _button.interactable = !isEmpty;
        if (!isEmpty)
        {
            // ここでは、あらかじめ定義しておいた配列からアイテム画像を取り出している
            // アイテムの種類が増えてきたら、アイテム画像をResourcesに入れておいてResources.Load()で読み出すなど、別の形で管理した方が楽になる
            image.sprite = itemSprites.First(x => x.itemType == _ownedItem.Type).sprite;
            number.text = "" + _ownedItem.Number;
        }
    }

    /// <summary>
    /// アイテムの種類とSpriteをインスペクタで紐付けられるようにするためのクラス
    /// クラスに[Serializable]をつけるとインスペクタで表示されるようになる
    /// </summary>
    [Serializable]
    public class ItemTypeSpriteMap
    {
        public Item.ItemType itemType;
        public Sprite sprite;
    }
}