using UnityEngine;
using UnityEngine.InputSystem;

[RequireComponent(typeof(PlayerInput))]
public class InputActionTest : MonoBehaviour
{
    private InputAction _move;
    private InputAction _jump;

    private void Start()
    {
        var input = GetComponent<PlayerInput>();
        // PlayerInputの「Default Map」で指定されているアクションマップを有効化
        input.currentActionMap.Enable();
        // アクションマップからアクションを取得するには FindAction() を使う
        _move = input.currentActionMap.FindAction("Move");
        _jump = input.currentActionMap.FindAction("Jump");
    }

    private void Update()
    {
        // Valueタイプのアクションは ReadValue<値の型>() で値を読み取る。値の型はInput Actionの設定に合わせる
        var moveValue = _move.ReadValue<Vector2>();
        if (moveValue.magnitude > 0f)
        {
            Debug.Log($"Moveアクションの値: {moveValue}");
        }

        // Buttonタイプのアクションは状態取得メソッドで状態を取得する
        if (_jump.WasPressedThisFrame())
        {
            Debug.Log("Jumpアクションが実行されたよ");
        }
    }
}